@@ -2,7 +2,7 @@ package HTML::Selector::XPath;
 
 use strict;
 use 5.008_001;
-our $VERSION = '0.09';
+our $VERSION = '0.10';
 
 require Exporter;
 our @EXPORT_OK = qw(selector_to_xpath);
@@ -14,13 +14,20 @@ sub selector_to_xpath {
     __PACKAGE__->new(shift)->to_xpath(@_);
 }
 
+# XXX: Identifiers should also allow any characters U+00A0 and higher, and any
+# escaped characters.
+my $ident = qr/(?![0-9]|-[-0-9])[-_a-zA-Z0-9]+/;
+
 my $reg = {
     # tag name/id/class
     element => qr/^([#.]?)([a-z0-9\\*_-]*)((\|)([a-z0-9\\*_-]*))?/i,
     # attribute presence
-    attr1   => qr/^\[([^\]]*)\]/,
+    attr1   => qr/^\[ \s* ($ident) \s* \]/x,
     # attribute value match
-    attr2   => qr/^\[\s*([^*~\|=\s:^\$]+)\s*([~\|*^\$]?=)\s*"([^"]+)"\s*\]/i,
+    attr2   => qr/^\[ \s* ($ident) \s*
+        ( [~|*^\$]? = ) \s*
+        (?: ($ident) | "([^"]+)" ) \s* \] /x,
+    badattr => qr/^\[/,
     attrN   => qr/^:not\((.*?)\)/i, # this should be a parentheses matcher instead of a RE!
     pseudo  => qr/^:([()a-z0-9_+-]+)/i,
     # adjacency/direct descendance
@@ -53,11 +60,11 @@ sub convert_attribute_match {
     } elsif ($op eq '|=') {
         "\@$left='$right' or starts-with(\@$left, '$right-')";
     } elsif ($op eq '^=') {
-        "starts-with(\@$left,'$3')";
+        "starts-with(\@$left,'$^N')";
     } elsif ($op eq '$=') {
-        "ends-with(\@$left,'$3')";
+        "ends-with(\@$left,'$^N')";
     } else { # exact match
-        "\@$left='$3'";
+        "\@$left='$^N'";
     }
 };
 
@@ -147,7 +154,7 @@ sub to_xpath {
 
         # Match attribute selectors
         if ($rule =~ s/$reg->{attr2}//) {
-            push @parts, "[", convert_attribute_match( $1, $2, $3 ), "]";
+            push @parts, "[", convert_attribute_match( $1, $2, $^N ), "]";
         } elsif ($rule =~ s/$reg->{attr1}//) {
             # If we have no tag output yet, write the tag:
             if (! $wrote_tag++) {
@@ -155,15 +162,19 @@ sub to_xpath {
                 $tag_index = $#parts;
             };
             push @parts, "[\@$1]";
+        } elsif ($rule =~ $reg->{badattr}) {
+            Carp::croak "Invalid attribute-value selector '$rule'";
         }
 
         # Match negation
         if ($rule =~ s/$reg->{attrN}//) {
             my $sub_rule = $1;
             if ($sub_rule =~ s/$reg->{attr2}//) {
-                push @parts, "[not(", convert_attribute_match( $1, $2, $3 ), ")]";
+                push @parts, "[not(", convert_attribute_match( $1, $2, $^N ), ")]";
             } elsif ($sub_rule =~ s/$reg->{attr1}//) {
                 push @parts, "[not(\@$1)]";
+            } elsif ($rule =~ $reg->{badattr}) {
+                Carp::croak "Invalid attribute-value selector '$rule'";
             } else {
                 my $xpath = selector_to_xpath($sub_rule);
                 $xpath =~ s!^//!!;
